package eu.cedarsoft.devtools;

import eu.cedarsoft.devtools.guice.GuiceIntegration;
import eu.cedarsoft.utils.CmdLine;
import org.jetbrains.annotations.NotNull;

/**
 * Starts the DevTools
 */
public class DevTools {
  public static void main( String[] args ) throws Exception {
    new DevTools().run( args );
  }

  private CmdLine cmdLine;
  private ModuleManager moduleManager;

  protected void run( String[] args ) throws Exception {
    //    DependencyInjectionIntegration integration = new SpringIntegration();
    DependencyInjectionIntegration integration = new GuiceIntegration();

    integration.getCommandLineContext().setArgs( args );

    cmdLine = integration.getCmdLine();
    moduleManager = integration.getModuleManager();

    cmdLine.out( MessageHandler.get( "welcome" ) );
    cmdLine.out( MessageHandler.get( "sponsored" ) );

    cmdLine.outNl();

    //Activate a module
    Module module = activateModule();

    //Now process the active module
    processModule( module );
  }

  /**
   * Activates a module
   */
  @NotNull
  private Module activateModule() {
    Module module = cmdLine.readSelection( MessageHandler.get( "select.module" ), moduleManager.getModules(), new Module.ModuleStringPresenter() );
    cmdLine.success( MessageHandler.get( "module.activated", module.getDescription() ) );
    return module;
  }

  /**
   * Processes the module
   *
   * @param module the module
   * @throws Exception
   */
  private void processModule( @NotNull Module module ) throws Exception {
    cmdLine.out( MessageHandler.get( "starting.module.0", module.getDescription() ) );
    cmdLine.outNl();
    try {
      module.process( cmdLine );
    } catch ( ModuleFailedException e ) {
      cmdLine.error( MessageHandler.get( "module.failed", e.getReturnType() ) );
    }
  }
}
