package eu.cedarsoft.devtools.guice;

import com.google.inject.AbstractModule;
import com.google.inject.Inject;
import com.google.inject.Provider;
import com.google.inject.Scopes;
import eu.cedarsoft.devtools.ModuleManager;
import eu.cedarsoft.devtools.modules.AddEmptySiteDescriptorModule;
import eu.cedarsoft.devtools.modules.AddExternalsDefinition;
import eu.cedarsoft.devtools.modules.AssemblyAttachedModule;
import eu.cedarsoft.devtools.modules.AssemblyModule;
import eu.cedarsoft.devtools.modules.CreateIdeaProjectModule;
import eu.cedarsoft.devtools.modules.CreateMavenDirectoryStructureModule;
import eu.cedarsoft.devtools.modules.EditExternalsDefinition;
import eu.cedarsoft.devtools.modules.EditIgnoreModule;
import eu.cedarsoft.devtools.modules.FastInstallModule;
import eu.cedarsoft.devtools.modules.IgnoreFileModule;
import eu.cedarsoft.devtools.modules.InstallFileToRemoteRepository;
import eu.cedarsoft.devtools.modules.InstallFileToRepository;
import eu.cedarsoft.devtools.modules.InstallModule;
import eu.cedarsoft.devtools.modules.ListRepositoryEntries;
import eu.cedarsoft.devtools.modules.MavenProjectCreationModule;
import eu.cedarsoft.devtools.modules.SubversionImportModule;
import eu.cedarsoft.devtools.modules.SwitchToNewUrlModule;

/**
 * Dev tools module
 */
public class DevToolsModule extends AbstractModule {
  protected void configure() {
    bind( ModuleManager.class ).toProvider( new Provider<ModuleManager>() {
      @Inject
      AddEmptySiteDescriptorModule addEmptySiteDescriptorModule;
      @Inject
      MavenProjectCreationModule mavenProjectCreationModule;
      @Inject
      CreateIdeaProjectModule createIdeaProjectModule;
      @Inject
      InstallModule installModule;
      @Inject
      FastInstallModule fastInstallModule;
      @Inject
      AssemblyModule assemblyModule;
      @Inject
      AssemblyAttachedModule assemblyAttachedModule;
      @Inject
      InstallFileToRepository installFileToRepository;
      @Inject
      InstallFileToRemoteRepository installFileToRemoteRepository;
      @Inject
      CreateMavenDirectoryStructureModule createMavenDirectoryStructureModule;
      @Inject
      SubversionImportModule subversionImportModule;
      @Inject
      ListRepositoryEntries listRepositoryEntries;
      @Inject
      SwitchToNewUrlModule switchToNewUrlModule;
      @Inject
      AddExternalsDefinition addExternalsDefinition;
      @Inject
      EditExternalsDefinition editExternalsDefinition;
      @Inject
      IgnoreFileModule ignoreFileModule;
      @Inject
      EditIgnoreModule editIgnoreModule;

      public ModuleManager get() {
        ModuleManager moduleManager = new ModuleManager();
        moduleManager.addModule( mavenProjectCreationModule );
        moduleManager.addModule( createIdeaProjectModule );
        moduleManager.addModule( installModule );
        moduleManager.addModule( fastInstallModule );
        moduleManager.addModule( assemblyModule );
        moduleManager.addModule( assemblyAttachedModule );
        moduleManager.addModule( installFileToRepository );
        moduleManager.addModule( installFileToRemoteRepository );
        moduleManager.addModule( createMavenDirectoryStructureModule );
        moduleManager.addModule( addEmptySiteDescriptorModule );

        moduleManager.addModule( subversionImportModule );
        moduleManager.addModule( listRepositoryEntries );
        moduleManager.addModule( switchToNewUrlModule );
        moduleManager.addModule( addExternalsDefinition );
        moduleManager.addModule( editExternalsDefinition );
        moduleManager.addModule( ignoreFileModule );
        moduleManager.addModule( editIgnoreModule );
        return moduleManager;
      }
    } ).in( Scopes.SINGLETON );
  }
}
