package eu.cedarsoft.devtools.guice;

import com.google.inject.AbstractModule;
import com.google.inject.Guice;
import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Module;
import com.google.inject.Provider;
import eu.cedarsoft.devtools.CommandLineContext;
import eu.cedarsoft.devtools.DependencyInjectionIntegration;
import eu.cedarsoft.devtools.ModuleManager;
import eu.cedarsoft.devtools.config.ConfigurationModule;
import eu.cedarsoft.devtools.config.DirectoriesModule;
import eu.cedarsoft.utils.ApplicationHomeAccess;
import eu.cedarsoft.utils.ApplicationName;
import eu.cedarsoft.utils.CmdLine;
import eu.cedarsoft.utils.DefaultApplicationHomeAccess;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;

/**
 * The Guice integration
 */
public class GuiceIntegration implements DependencyInjectionIntegration {
  protected final Injector injector;

  public GuiceIntegration() {
    injector = Guice.createInjector( getApplicationModule(), getCmdLineModule(), new ConfigurationModule(), new DirectoriesModule(), new SubversionModule(), new MavenModule(), new DevToolsModule() );
  }

  @NotNull
  protected Module getApplicationModule() {
    return new AbstractModule() {
      @Override
      protected void configure() {
        bindConstant().annotatedWith( ApplicationName.class ).to( "devtools" );
        bind( ApplicationHomeAccess.class ).toProvider( new Provider<ApplicationHomeAccess>() {
          @Inject
          @ApplicationName
          String applicationName;

          public ApplicationHomeAccess get() {
            try {
              return new DefaultApplicationHomeAccess( applicationName );
            } catch ( IOException e ) {
              throw new RuntimeException( e );
            }
          }
        } );
      }
    };
  }

  @NotNull
  protected Module getCmdLineModule() {
    return new CmdLineModule();
  }

  @NotNull
  public CommandLineContext getCommandLineContext() {
    return injector.getInstance( CommandLineContext.class );
  }

  @NotNull
  public CmdLine getCmdLine() {
    return injector.getInstance( CmdLine.class );
  }

  @NotNull
  public ModuleManager getModuleManager() {
    return injector.getInstance( ModuleManager.class );
  }
}
