package eu.cedarsoft.devtools.guice;

import com.google.inject.AbstractModule;
import com.google.inject.Inject;
import com.google.inject.Provider;
import com.google.inject.Scopes;
import com.jgoodies.binding.beans.BeanAdapter;
import eu.cedarsoft.devtools.DirectorySupport;
import eu.cedarsoft.devtools.SubversionBin;
import eu.cedarsoft.devtools.SubversionConfiguration;
import eu.cedarsoft.devtools.SubversionSupport;
import eu.cedarsoft.utils.configuration.ConfigurationAccess;
import eu.cedarsoft.utils.configuration.ConfigurationBinding;
import eu.cedarsoft.utils.configuration.ConfigurationManager;
import eu.cedarsoft.utils.configuration.DefaultValueProvider;
import eu.cedarsoft.utils.configuration.EmptyDefaultValueProvider;
import org.apache.commons.configuration.Configuration;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Map;

/**
 * The Subversion Module
 */
public class SubversionModule extends AbstractModule {
  @Override
  protected void configure() {
    bindConfiguration();
    bindSubversionBin();
    bindSubversionSupport();
  }

  private void bindSubversionSupport() {
    bind( SubversionSupport.class ).toProvider( new Provider<SubversionSupport>() {
      @Inject
      @SubversionBin
      String subersionBin;

      @Inject
      DirectorySupport directorySupport;

      @Inject
      DefaultValueProvider defaultValueProvider;

      @Inject
      @SubversionConfiguration
      Configuration configuration;

      @NotNull
      public SubversionSupport get() {
        SubversionSupport subversionSupport = new SubversionSupport( directorySupport, subersionBin );
        BeanAdapter<SubversionSupport> beanAdapter = new BeanAdapter<SubversionSupport>( subversionSupport, true );

        {
          ConfigurationAccess<List> configurationAccess = new ConfigurationAccess<List>( configuration, List.class, SubversionSupport.PROPERTY_REPOSITORY_URLS, EmptyDefaultValueProvider.INSTANCE );
          ConfigurationBinding.bind( configurationAccess, beanAdapter.getValueModel( SubversionSupport.PROPERTY_REPOSITORY_URLS ) );
        }
        {
          ConfigurationAccess<Map> configurationAccess = new ConfigurationAccess<Map>( configuration, Map.class, SubversionSupport.PROPERTY_OLD_REPOSITORY_URLS, EmptyDefaultValueProvider.INSTANCE );
          ConfigurationBinding.bind( configurationAccess, beanAdapter.getValueModel( SubversionSupport.PROPERTY_OLD_REPOSITORY_URLS ) );
        }
        return subversionSupport;
      }
    } ).in( Scopes.SINGLETON );
  }

  private void bindSubversionBin() {
    bind( String.class ).annotatedWith( SubversionBin.class ).toProvider( new Provider<String>() {
      @Inject
      DefaultValueProvider defaultValueProvider;

      @Inject
      @SubversionConfiguration
      Configuration configuration;

      public String get() {
        ConfigurationAccess<String> access = new ConfigurationAccess<String>( configuration, String.class, "subversion.bin", defaultValueProvider );
        return access.resolve();
      }
    } );
  }

  private void bindConfiguration() {
    bind( Configuration.class ).annotatedWith( SubversionConfiguration.class ).toProvider( new Provider<Configuration>() {
      @Inject
      ConfigurationManager configurationManager;

      @NotNull
      public Configuration get() {
        return configurationManager.getModuleConfiguration( SubversionSupport.class );
      }
    } ).in( Scopes.SINGLETON );
  }
}
